#include "pdata.h"
#include <qdebug.h>
#include <math.h>

PData::PData(uchar *data, QSize size)
{
        width = size.width();
        height = size.height();

        dataMatrix = new uchar [width*height];
//       uchar *dataMatrixPtr = dataMatrix;
//        uchar  *imageDataPtr = data;
    /*
     * 4 bytes per pixel: 3 colors + the alpha channel
     */
        for (int i=0; i<width*height;i++)
                        dataMatrix[i] = data[4*i];

        projection = new double[width];
        filteredProjection = new double[width];
        gaussianData = new double[width];
        aoiDataMatrix = new uchar [width*height];
        for (int i=0;i<width*height;i++)
            aoiDataMatrix[i] = dataMatrix[i];
        xLow = 0;
        xHigh = width;

}

PData::~PData()
{
        delete dataMatrix;
        delete aoiDataMatrix;
        delete projection;
        delete filteredProjection;
        delete gaussianData;
}

void PData::selectAoi(int aoiX1, int aoiY1, int aoiX2, int aoiY2)
{
    int yLow,yHigh;
    int i,j;
    aoiY1 -=10;
    aoiY2 -=10;
    qDebug() << " aoi: " << aoiX1 << " " << aoiY1 <<" " << aoiX2 << " " << aoiY2<< endl;
    if (aoiX2 < aoiX1) {
        xLow = aoiX2;
        xHigh = aoiX1;
    }
    else {
        xLow = aoiX1;
        xHigh = aoiX2;
   }
   if (aoiY2 < aoiY1) {
        yLow = aoiY2;
        yHigh = aoiY1;
    }
    else {
        yLow = aoiY1;
        yHigh = aoiY2;
    }
//    qDebug() << "x1,y1,x2,y2 " << xLow << " " << yLow << " " << xHigh << " " << yHigh << endl;

    for (i=0; i<height;i++)
        for (j=0; j<width;j++)
            if ((i < yLow) || (i > yHigh) || (j<xLow) || (j>=xHigh))
                aoiDataMatrix[i*width+j] = 0;
            else
                aoiDataMatrix[i*width+j] = dataMatrix[i*width+j];
//                aoiDataMatrix[i*width+j] = 127;

}

void PData::calcProjection()
{
        int i,j;

        if (aoiDataMatrix == 0)
                qDebug() <<"No data\n" << endl;
        //
        // first clear out the array of projection data
        //
        for (i=0; i< width; i++)
                projection[i] = 0.0;
        //
        // the calculate the projection
        //

        for (i=0; i< height; i++) {
            for (j=0; j < width; j++) {
                        projection[j] += (double) aoiDataMatrix[i*width+j];
                }
       }
       lowPass();
 /*
  * try offset subtraction
  */
       if (filterData) {
           slope = (filteredProjection[xHigh-3]-filteredProjection[xLow+3])/(xHigh-3 - xLow+3);
           offs = filteredProjection[xLow+3] - slope * xLow;       }
       else {
            slope = (projection[xHigh-1]-projection[xLow])/(xHigh-1 - xLow);
            offs = projection[xLow] - slope * xLow;
        }
/*
        if (subOffset)
            qDebug() << "sub offset" << endl;
        else
             qDebug() << "no offset subtraction" << endl;
*/
        if (subOffset) {
            for (int i=xLow; i<xHigh; i++) {
                projection[i] = projection[i] - (slope * ((double) i) + offs);
                filteredProjection[i] = filteredProjection[i] - (slope * ((double) i) + offs);
            }
        }
        return;
}

double * PData::lowPass()
{
        const double A_COEFF = 0.05;
        const double B_COEFF = 0.1;
        const double C_COEFF = 0.2;
        const double D_COEFF = 0.3;

        filteredProjection[0] = projection[0];
        filteredProjection[1] = projection[1];
        filteredProjection[2] = projection[2];
        int i;
        for (i=3;i<width-3;i++) {
                filteredProjection[i] = projection[i-3]*A_COEFF + projection[i-2]*B_COEFF + projection[i-1]*C_COEFF +
                                        projection[i]*D_COEFF +
                                        projection[i+1]*C_COEFF + projection[i+2]*B_COEFF + projection[i+3]*A_COEFF;
        }

        filteredProjection[i]   = projection[i];
        filteredProjection[i+1] = projection[i+1];
        filteredProjection[i+2] = projection[i+2];
        return filteredProjection;
}

int PData::getWidth()
{
        return width;
}

int PData::getHeight()
{
        return height;
}

double PData::maximum()
{
        double max = 0.0;
        for (int i= 0; i<width;i++)
                if (projection[i] > max)
                        max = projection[i];
        return max;
}

double PData::firstMoment()
{
    double sum = 0.0;
    double sumY = 0.0;
    for (int i=xLow; i<xHigh;i++) {
        if (filterData) {
            sum += ((double) i) * filteredProjection[i];
            sumY += filteredProjection[i];
        }
        else {
           sum += ((double) i) * projection[i];
           sumY += projection[i];
       }
    }
//    qDebug() << "first Moment " << sum/sumY << endl;
    return sum/sumY;
}

double PData::twoSigma()
{
    double sum = 0.0;
    double firstMom = firstMoment();
    double sumY = 0.0;
    double x;
    for (int i=xLow; i<xHigh;i++) {
        x = (double) i;
        if (filterData) {
            sum  += (x-firstMom)*(x-firstMom)*filteredProjection[i];
            sumY += filteredProjection[i];
        }
        else {
            sum  += (x-firstMom)*(x-firstMom)*projection[i];
            sumY += projection[i];
        }
    }

    qDebug() << "sum sumY " << sum << sumY <<endl;
    return 2*sqrt(sum/sumY);
}


double * PData::gaussian()
{
    double sigma = twoSigma()/2.0;
    double factor = 1.0/(sqrt(2*M_PI)*sigma);
    double b = firstMoment();
    double x,exponent;
    double scal=integral();
    qDebug() << " b,sigma,scal " << b << " " << sigma << " " << scal << endl;
    for (int i=0;i<width;i++) {
        x = i;
        exponent = (x-b)*(x-b)/(2*sigma*sigma);
        gaussianData[i] = scal*factor*exp(-exponent);
//        printf("%f\n",gaussianData[i]);
    }
    return gaussianData;
}

double PData::integral()
{
    double sum = 0;
    for (int i=xLow; i<xHigh; i++)
        sum += projection[i];
    return sum;
}

