#include "qprofiles.h"
#include "ui_qprofiles.h"
#include "ui_projectionDialog.h"

#include <qpixmap.h>
#include <qdebug.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include "pimage.h"

QProfiles::QProfiles(QWidget *parent)
    : QMainWindow(parent), ui_qprofiles(new Ui::QProfiles),
                           ui_projectionDialog(new Ui::ProjectionDialog)
{
    ui_qprofiles ->setupUi(this);
    /*
     * Creating a special label is needed in order to recuperate mouse events
     * This allows implementation of "area of interest" features
     */

    imageLabel = new PImage((QWidget *)ui_qprofiles->imageBox);
    if (!setPixmap(QString(":/icons/images/uspas.png"))) {
        QMessageBox msgBox;
        msgBox.setText(tr("Cannot open default Pixmap: :/images/uspas.png") );
        msgBox.setIcon(QMessageBox::Critical);
        msgBox.exec();
        exit(-1);
    }
    imageLabel->setGeometry(QRect(10, 20, 650, 550));
    imageLabel -> setPixmap(imagePixmap);
    aoiPixmap = imagePixmap;

    QImage pImage = imagePixmap.toImage();
    if (pImage.format() != QImage::Format_RGB32)
        pImage.convertToFormat(QImage::Format_RGB32);
    QSize size = pImage.size();
    uchar *pixelData = pImage.bits();
    imageData = new PData(pixelData,size);

    imageData -> subOffset = ui_qprofiles->actionSubtract_Offset->isChecked();
    imageData -> filterData = ui_qprofiles->actionFilter_Data->isChecked();

    /*
     * hook up the actions to their code
     */
    connect(ui_qprofiles->actionQuit, SIGNAL(triggered()),this,SLOT(fileQuit()));
    connect(ui_qprofiles->actionSave_Projection, SIGNAL(triggered()),this,SLOT(fileSaveHor()));
    connect(ui_qprofiles->actionOpen_Image, SIGNAL(triggered()),this,SLOT(fileOpen()));
    connect(ui_qprofiles->aoiButton, SIGNAL(clicked()),this,SLOT(aoiSelected()));
    connect(ui_qprofiles->actionHor_Projection, SIGNAL(triggered()),this,SLOT(projection()));
    connect(ui_qprofiles->actionSubtract_Offset, SIGNAL(triggered(bool)),this,SLOT(toggleSubtractOffset(bool)));
    connect(ui_qprofiles->actionFilter_Data, SIGNAL(triggered(bool)),this,SLOT(toggleFilterData(bool)));
    connect(imageLabel, SIGNAL(resetPixmap()),this,SLOT(resetAoi()));
    connect(ui_qprofiles->actionAbout_QProfiles,SIGNAL(triggered()),this,SLOT(displayAbout()));
    connect(ui_qprofiles->actionWhatsThis, SIGNAL(triggered()),this,SLOT(whatsThis()));
    qDebug() << "after all connects" << endl;
    curve = 0;
    projDialog = 0;

}

QProfiles::~QProfiles()
{

    delete ui_qprofiles;
    delete imageData;
    if (curve!= 0)
        delete curve;
    if (projDialog != 0)
        delete projDialog;
}
void QProfiles::fileSaveHor()
{
        // this slot is called whenever the File->Save Hor menu is selected
    QString fileName = QFileDialog::getSaveFileName(this,
        tr("Save Projection"), "images", tr("Projections (*.dat)"));
    qDebug() << "Filename: " << fileName << endl;

    if (QFile::exists(fileName)) {
        QString question=tr("File ") + fileName + tr(" exists! Do you want to replace it") + QString("?");
        QMessageBox msgBox;
        msgBox.setText(question);
        msgBox.setIcon(QMessageBox::Question);
        msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
        msgBox.exec();       
    }
    QFile outputFile(fileName);
    if (!outputFile.open(QIODevice::WriteOnly)) {
        QMessageBox msgBox;
        msgBox.setText(tr("Cannot open File") );
        msgBox.setIcon(QMessageBox::Critical);
    }
    QTextStream textStream(&outputFile);
    double *projection = imageData -> projection;
    QString dataString;
    int width = imageData -> width;
    for (int i=0; i<width; i++) {
         dataString.sprintf("%f\n",projection[i]);
         textStream << dataString;
    }

    outputFile.close();
    return;
}

void QProfiles::fileQuit()
{
    QCoreApplication::quit();
}
void QProfiles::displayAbout()
{
    QString titleString(tr("About QProfiles"));
    QString aboutString1(tr("QPepperpot is a Qt-4.5 application for evaluation of light spot images\n"));
    QString aboutString2(tr("The program has been written for USPAS,\n"));
    QString aboutString3(tr("the US accelerator school, dedicated to beam diagnostics.\n"));
    QString aboutString4(tr("Albuquerque 22.6 - 26.6\n"));
    QString authorString(tr("Author: Uli Raich, CERN BE/BI (uli.raich@cern.ch)"));
    QMessageBox::about(this,titleString,aboutString1+aboutString2+aboutString3+aboutString4+authorString);
}

void QProfiles::whatsThis()
{
    QWhatsThis::enterWhatsThisMode();
}

void QProfiles::fileOpen()
{
    QString fileName = QFileDialog::getOpenFileName(this,
     tr("Open Image"), "images", tr("Image Files (*.png *.jpg *.bmp)"));

    qDebug() << "Filename" << fileName << endl;
    if (fileName.isEmpty())
        return;
     QPixmap pixmap(fileName);
     imagePixmap = pixmap;
     if (imagePixmap.isNull()) {
        QMessageBox msgBox;
        msgBox.setText(fileName + tr(" is not a valid pixmap file"));
        msgBox.setIcon(QMessageBox::Critical);
        msgBox.exec();
        return;
 }

    if (imageData != 0)
        delete imageData;    
    QImage pImage = imagePixmap.toImage();
    if (pImage.format() != QImage::Format_RGB32)
        pImage.convertToFormat(QImage::Format_RGB32);
    QSize size = pImage.size();
    uchar *pixelData = pImage.bits();
    imageData = new PData(pixelData,size);
     if (imageData -> subOffset)
            qDebug() << "qprofiles sub offset" << endl;
        else
             qDebug() << "qprofiles no offset subtraction" << endl;

    aoiPixmap = imagePixmap;
    (ui_qprofiles -> imageBox) ->setTitle(QString(tr("file: ") + fileName));
    if (projDialog!=0)
    {
        delete projDialog;
        projDialog = 0;
        projection();
    }
    imageLabel -> setPixmap(imagePixmap);
}

bool QProfiles::setPixmap(QString fileName)
{
    QPixmap pixmap(fileName);
    if (pixmap.isNull())
        return false;
    imageLabel->setPixmap(pixmap);
    imagePixmap = pixmap;
    imageFileName = fileName;
    return true;
}

void QProfiles::aoiSelected()
{
    qDebug() << "aoi selected" << endl;
    QRect aoi = imageLabel -> getAoi();
    imageData -> selectAoi(aoi.x(),aoi.y(),aoi.width(),aoi.height());

    QImage pImage = aoiPixmap.toImage();
    if (pImage.format() != QImage::Format_RGB32)
        pImage.convertToFormat(QImage::Format_RGB32);
    QSize size = pImage.size();

    qDebug() << "rectangle: " << aoi.x() << " "<< aoi.y() <<endl;
    qDebug() << "size: " << size.width() << " "<< size.height() <<endl;

    uchar *pixelData = pImage.bits();
    for (int i=0; i < imageData->width*imageData->height; i++) {
        pixelData[4*i] = imageData -> aoiDataMatrix[i];
        pixelData[4*i+1] = imageData -> aoiDataMatrix[i];
        pixelData[4*i+2] = imageData -> aoiDataMatrix[i];
        pixelData[4*i+3] = 0xff;
    }

    QPixmap newPixmap = QPixmap::fromImage(pImage);
    imageLabel -> setPixmap(newPixmap);

    aoiPixmap = newPixmap;

    if (projDialog!=0) {
        delete projDialog;
        projDialog = 0;
        projection();
    }

}

void QProfiles::projection()
{
    int width = imageData -> width;
    double max = imageData ->maximum();

    qDebug() << "width: "<< width << "max: " << max << endl;

    if (projDialog==0) {
        qDebug() << "create a new dialog box"<< endl;
        projDialog = new QDialog( this );
        projDialog->setAttribute( Qt::WA_DeleteOnClose );
        projDialog->setWindowTitle(QString( "Hor Projection" ));
        ui_projectionDialog -> setupUi(projDialog);
        ui_projectionDialog -> projectionPlot -> setTitle("Horizontal Projection");
        ui_projectionDialog -> projectionPlot -> setAxisTitle(QwtPlot::xBottom,"Pixel Number");
        ui_projectionDialog -> projectionPlot ->setWhatsThis(QString(tr("The projection, offset and Gaussian curve plots")));
        ui_projectionDialog -> projectionPlot ->setToolTip(QString(tr("ProjectionPlot")));
        projDialog -> show();
        connect(projDialog, SIGNAL(destroyed()), this, SLOT(projDialogDestroyed()));
    }
    double xValues[width];
    double *xValPtr;
    xValPtr = xValues;
    for (int i=0; i< width; i++)
                *xValPtr++ = (double) i;
    QwtPlot *projPlot = ui_projectionDialog -> projectionPlot;
    imageData -> calcProjection();
    double *projection = imageData ->projection;

    projPlot -> setAxisScale(QwtPlot::xBottom,0.0,(double)width);
    max = imageData -> maximum();
    //
    // add 10%, just in case
    //

    max = max + max/10;
    qDebug() << "max " << max <<endl;

    projPlot -> setAxisScale(QwtPlot::yLeft,0.0,max);

    if (curve!=0)
        delete curve;

    curve = new QwtPlotCurve("Hor Projection");
    curve -> setData(xValues,projection,width);
    curve -> setPen(QPen(Qt::blue));
    curve -> attach(projPlot);

    double *control = new double[width];
    for (int i=0; i<width;i++)
        control[i]=0.0;
    int fm = (int) imageData -> firstMoment();
    control[fm] = projection[fm];
    int sm = (int) imageData -> twoSigma();
    control[fm - sm/2] = projection[fm];
    control[fm + sm/2] = projection[fm];

    double *gaussData = imageData ->gaussian();

    QwtPlotCurve *controlCurve = new QwtPlotCurve("Hor Projection");
    controlCurve -> setData(xValues,control,width);
    controlCurve -> setPen(QPen(Qt::black));
    controlCurve -> attach(projPlot);

    QwtPlotCurve *gaussCurve = new QwtPlotCurve("Gauss Fit");
    gaussCurve -> setData(xValues,gaussData,width);
    gaussCurve -> setPen(QPen(Qt::red));
    gaussCurve -> attach(projPlot);

    if (imageData -> filterData) {
        double *filteredProjection = imageData ->filteredProjection;
//        if (filteredCurve != 0)
//            delete filteredCurve;

        filteredCurve = new QwtPlotCurve("Filtered Projection");
        filteredCurve -> setData(xValues,filteredProjection,width);
        filteredCurve -> setPen(QPen(Qt::green));
        filteredCurve -> attach(projPlot);
    }

    if (!(imageData -> subOffset)) {
        offsetCurve = new QwtPlotCurve("Offset");
        double offset[width];
        qDebug() << "Slope + offset: " << imageData -> slope << " " << imageData -> offs << endl;

        for (int i=0;i<width;i++)
            offset[i] = imageData -> slope * ((double) i) + imageData -> offs;

        offsetCurve -> setData(xValues,offset,width);
        offsetCurve -> setPen(QPen(Qt::magenta));
        offsetCurve -> attach(projPlot);
    }
    QString widthString;
    widthString.sprintf("%f",imageData->twoSigma());
    (ui_projectionDialog -> widthText) -> setText(widthString);
    return;
}

void QProfiles::resetAoi()
{
    qDebug() << "resetAoi" << endl;
    imageLabel -> setPixmap(imagePixmap);
    aoiPixmap = imagePixmap;

    QImage pImage = imagePixmap.toImage();
    if (pImage.format() != QImage::Format_RGB32)
        pImage.convertToFormat(QImage::Format_RGB32);
    QSize size = pImage.size();
    uchar *pixelData = pImage.bits();

    if (imageData!=0)
        delete imageData;
    imageData = new PData(pixelData,size);

//    qDebug() << "deleting dialog box" << endl;
    if (projDialog !=0 ) {
        delete projDialog;
        projection();
    }

}
void QProfiles::projDialogDestroyed()
{
    curve = 0;
    projDialog = 0;

//    qDebug() << "Projection Dialog destroyed" << endl;
}

void QProfiles::toggleSubtractOffset(bool on)
{
    qDebug() << "toggle subtract offset" << endl;
    imageData -> subOffset=on;

    if (projDialog!=0) {
        delete projDialog;
        projDialog = 0;
        projection();
    }
}

void QProfiles::toggleFilterData(bool on)
{
    qDebug() << "toggle filter data" << endl;
    imageData -> filterData=on;

    if (projDialog!=0) {
        delete projDialog;
        projDialog = 0;
        projection();
    }
}
